function [ zp ] = Main_loocv_sli( data, modelsli, Param_sli )
%   This function generates SLI predictions and standard deviations at the
%   sampling sites using leave-one-out cross validation (LOO-CV)
%   The SLI estimation function Main_sli_mle_needs to run first in order to
%   estimate the SLI model. 
%================ INPUT PARAMETERS ========================================
%
%   data:       Structure containing fields data.locs and data.val.
%   data.locs:	Training set locations (array N x dd)
%   data.val:	Field values at the training set points (N x 1)
%   modelsli:	Structure containing SLI model (from Main_sli_mle)
%   Param_sli:  SLI parameters [mz lambda c1 mu] (derived from Main_sli_mle)
%================ OUTPUT PARAMETERS =======================================
%
%   zp:         Structure containing fields zp.est and zp.std
%   zp.est:     SLI-based LOO-CV estimates
%   zp.std:     SLI-based LOO-CV standard deviations
%
%==========================================================================
%
%  Copyright (C) 2020  Dionisis Hristopulos, Andreas Pavlides, Vasiliki
%  Agou
%   email: dchristopoulos@ece.tuc.gr
%
%  This program is free software; you can redistribute it and/or modify it
%  under the terms of the GNU General Public License as published by the
%  Free Software Foundation (version 2 of the License or  later version).
%
%  This program is distributed WITHOUT ANY WARRANTY.
%  See the GNU General Public License for more details.
%==========================================================================
kernel = modelsli.kernel;
K = modelsli.nei;

locs = data.locs;
dd = size(locs, 2); % Dimension of space
z = data.val; 
N = length(z);

for n=1:N
    %   Remove the datum n
    zsi = z;
    zsi(n) = [];
    locsi = locs;
    locsi(n, :) = [];
    
    %   Remove the n-th elements from the model structure
    Jn = modelsli.J;
    Jsubn = modelsli.Jsub;
    distmsn = modelsli.distms;
    h1n = modelsli.h1;
    Den = modelsli.Den;
    Jn(:,n) = [];
    Jn(n,:) = [];
    distmsn{1}(:,n)=[];
    distmsn{1}(n,:)=[];
    distmsn{2}(n,:)=[];
    h1n(n,:) = [];
    Jsubn{1}(:,n)=[];
    Jsubn{1}(n,:)=[];
    Jsubn{2}(:,n)=[];
    Jsubn{2}(n,:)=[];
    
    modeln.J = Jn;
    modeln.Jsub = Jsubn;
    modeln.distms = distmsn;
    modeln.h1 = h1n;
    modeln.Den = Den;
    
    [Zphat, sli_var ] = Predict_SLI(dd, locsi, zsi, locs(n, :), ...
        kernel, K, modeln, Param_sli );
    zp.est(n,1) = Zphat;
    zp.std(n,1) = sqrt(sli_var);
    
end;

%   Note: the variable zp cannot be named Zhat because this conflicts
%   with variable names in crosval_new


