function [ Zphat, sli_var ] = Predict_SLI(dd, locs, zs, plocs, kernel, K, model, Param )
%================ INPUT PARAMETERS ========================================
%
%   dd:         Dimensionality
%   LOCS:       Training set locations (array N x dd)
%   ZS:         Field values at the training set points
%   PLOCS:      Validation set locations (array P x dd )
%   KERNEL:     Kernel function used
%   K:          Neighbor order 
%   MODEL:      Sample precision and distance matrices and the vector of 
%               bandwidths h1
%   PARAM:      SLI parameters [mz lambda c1 mu]
%
%================ OUTPUT PARAMETERS =======================================
%   PROGRAM OUTPUTS INCLUDE THE FOLLOWING
%
%   ZPHAT:       SLI Estimates at validation locations
%   SLI_VAR:     Variance of SLI estimates at prediction points
%
%==========================================================================
%   OTHER FUNCTIONS CALLED: calc_dist_cross_g, estim_JP_gra
%==========================================================================
%   TRAINING - VALIDATION SET POINT DISTANCES
%==========================================================================

mz = Param(1);
lambda = Param(2);
Params= Param(3:end);

zss = zs - mz ; 

[ rsp, hx1] = calc_dist_cross_g(dd, locs, plocs, K);

distmsp{1} = rsp;
distmsp{2} = hx1;

h1 = model.h1;
Den = model.Den;

[JPS, JPP] = estim_JP_gra(Params, distmsp, h1, Den, kernel);

Zphat = mz*ones(size(plocs, 1), 1) - ( JPS * zss(:) ) ./  JPP;

sli_var = lambda ./ JPP;

